using System;
using System.Runtime.InteropServices;
using System.Threading;

namespace OptimusMiniUsageExample
{

	// OptimusMini.dll interface class.
	class IOptimusMini
	{
		const string DRIVER="OptimusMini.dll";

		public const int SCREEN_SIZE=96;
		public const int SCREEN_BUFFER_SIZE=SCREEN_SIZE*SCREEN_SIZE*3;

		public delegate void OnKeyDownCallbackDelegate(int key);
		public delegate void OnDeviceStateChangedCallbackDelegate(int state);

		[DllImport(DRIVER)]
		public static extern void Init();

		[DllImport(DRIVER)]
		public static extern void Close();

		[DllImport(DRIVER)]
		public static extern int IsAlive();

		[DllImport(DRIVER)]
		public static extern int SendCommand(byte c1, byte c2);

		[DllImport(DRIVER)]
		public static extern int SendCommandBlocking(byte c1, byte c2);

		[DllImport(DRIVER)]
		public static extern int ShowPicture(int p_nButton, byte[] p_lpData);
		
		[DllImport(DRIVER)]
		public static extern int ShowPictureBlocking(int p_nButton, byte[] p_lpData);

		[DllImport(DRIVER)]
		public static extern int RegisterEventHandler(OnKeyDownCallbackDelegate dk, OnDeviceStateChangedCallbackDelegate ds);
	}

	class UsageExample 
	{
		static void SetColor(byte[] img, byte r, byte g, byte b)
		{
			for (int i=0; i<IOptimusMini.SCREEN_BUFFER_SIZE; i+=3)
			{
				img[i+0]=r;
				img[i+1]=g;
				img[i+2]=b;
			}
		}

		static void OnKeyDownCallbackHandler(int key)
		{
			Console.Write("Keydown {0}\n", key);
		}

		static void OnDeviceStateChangedCallbackHandler(int state)
		{
			Console.Write( state > 0 ? "Connected.\n" : "Disconnected.\n");
		}
		
		static void Main( string[] args ) 
			{
			byte[] img = new byte[IOptimusMini.SCREEN_BUFFER_SIZE];

			/* register callback handlers if you want to process keypresses */
			IOptimusMini.OnKeyDownCallbackDelegate oKD = new IOptimusMini.OnKeyDownCallbackDelegate(OnKeyDownCallbackHandler);
			IOptimusMini.OnDeviceStateChangedCallbackDelegate oDSC = new IOptimusMini.OnDeviceStateChangedCallbackDelegate(OnDeviceStateChangedCallbackHandler);
			IOptimusMini.RegisterEventHandler(oKD, oDSC);

			Console.Write("Initing... ");
			IOptimusMini.Init();
		
			for(int i=0;IOptimusMini.IsAlive()==0;i++)
			{
				Thread.Sleep(100);
				
				if (i>100)
				{
					Console.Write("Not found.\n");
					IOptimusMini.Close();
					return;
				}
			}

			// swithing on 
			IOptimusMini.SendCommand( 2,0 );
			// setting brightness
			IOptimusMini.SendCommand( 9, 60 );

			Console.Write("Showing sample images.\n");

			SetColor(img,255,0,0);
			IOptimusMini.ShowPictureBlocking(1,img);
			SetColor(img,0,255,0);
			IOptimusMini.ShowPictureBlocking(2,img);
			SetColor(img,0,0,255);
			IOptimusMini.ShowPictureBlocking(3,img);

			Console.Write("Testing keypresses for 10 second.\n");

			for(int i=0;IOptimusMini.IsAlive()==1 && i<100;i++)
			{
				Thread.Sleep(100);
			}
			
			IOptimusMini.Close();

			/* callback handlers should be referenced all the time OptimusMini is used */
			GC.KeepAlive(oKD);
			GC.KeepAlive(oDSC);
		}
	}
}